@@ -1,6 +1,7 @@
-# Copyright (C) 2010, 2011 Linaro Limited
+# Copyright (C) 2010, 2011, 2015 Linaro Limited
 #
 # Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
+#         Neil Williams <neil.williams@linaro.org>
 #
 # This file is part of django-testscenarios.
 #
@@ -40,6 +41,7 @@ class TestModel(models.Model):
     Model for testing database configuration/initialization
     """
     field = models.CharField(max_length=10, null=True)
+    broken = models.IntegerField(default=1)
 
 
 class ScenarioParametersAreVisibleChecks(object):
@@ -59,6 +61,8 @@ class PlainDatabaseChecks(object):
         self.assertEqual(TestModel.objects.all().count(), 0)
         obj = TestModel.objects.create()
         self.assertEqual(TestModel.objects.all().count(), 1)
+        obj.delete()
+        self.assertEqual(TestModel.objects.all().count(), 0)
 
     def test_database_is_empty_at_start_of_test_first(self):
         self._do_check_for_database_state()
@@ -69,6 +73,7 @@ class PlainDatabaseChecks(object):
 
 class TransactionChecks(object):
 
+    @transaction.atomic
     def _create_object(self):
         self.obj = TestModel.objects.create(field=None)
         self.pk = self.obj.pk
@@ -76,28 +81,25 @@ class TransactionChecks(object):
     def _reload_object(self):
         self.obj = TestModel.objects.get(pk=self.pk)
 
-    def _commit_and_reload(self):
-        transaction.commit()
-        self._reload_object()
-
-    def _rollback_and_reload(self):
-        transaction.rollback()
-        self._reload_object()
-
     def test_transaction_handling(self):
-        transaction.enter_transaction_management()
-        transaction.managed(True)
-        try:
+        with transaction.atomic():
             self._create_object()
-            self._commit_and_reload()
-            self.assertEqual(self.obj.field, None)
-            self.obj.field = "something"
-            self.obj.save()
-            self._rollback_and_reload()
-            self.assertEqual(self.obj.field, None)
-        finally:
-            transaction.rollback()
-            transaction.leave_transaction_management()
+        self._reload_object()
+        self.assertEqual(self.obj.field, None)
+        self.obj.field = "something"
+        self.obj.save()
+        try:
+            self.obj.broken = 'not an integer'
+        except ValueError:
+            pass
+        try:
+            with transaction.atomic():
+                self.obj.save()
+        except ValueError:
+            pass
+        self._reload_object()
+        self.assertEqual(self.obj.broken, 1)
+        self.assertNotEqual(self.obj.field, None)
 
 
 # Non-transaction tests
